unit GButtons;

INTERFACE

const
   MaxNumButtons = 20;
   MaxNumCButtons = 10;

type
	STR40 = string[40];
   Polarity = (plus,minus);
   ProcType = procedure;

   IconObject = object
      l,t,r,b : integer;
      selected : boolean;
      Cmd : ProcType;
      procedure Define (Left,Top,Right,Bottom : integer;
                        Command : ProcType);
   end;

	ButtonObject = object (IconObject)
		x,y,w,h,c : integer;
		BtnTxt : STR40;
		procedure Draw;
		procedure Create (Ptx,Pty,Wd,Ht,Color:integer;
                        TxtLabel:STR40;
                        Command : ProcType);
      procedure HiLite;
	end;

   CButtonObject = object (ButtonObject)
      p : polarity;
      procedure Draw;
      procedure Create (Ptx,Pty,Wd,Ht,Color:integer;
                        pol: polarity;
                        Command : ProcType);
   end;

var                              { Global Variables }
   NumButtons, NumCButtons, NumDButtons : integer;
   Button : array[1..MaxNumButtons] of ButtonObject;
   CButton : array[1..MaxNumCButtons] of CButtonObject;

procedure DetectHits;

IMPLEMENTATION

uses Crt, Graph, MousePac;

procedure IconObject.Define (Left,Top,Right,Bottom : integer;
                             Command : ProcType);
   begin
      l   := Left;
      r   := Right;
      t   := Top;
      b   := Bottom;
      selected := FALSE;
      Cmd := Command;
   end;


procedure ButtonObject.Draw;
   var
      i,AlignX,AlignY : integer;
      TempStr  : string[2];
   const
      depth = 3;

   begin
		SetViewPort (x,y,x+w,y+h,ClipOn);
		SetFillStyle (SolidFill,C);
		Bar (0,0,w,h);
		SetColor (Black);
		Rectangle (0,0,w,h);
		Rectangle (2*depth,depth,w-2*Depth,h-Depth);
      Line (0,0,2*depth,depth);
		Line (0,h,2*depth,h-depth);
		Line (w,0,w-2*depth,depth);
		Line (w,h,w-2*depth,h-depth);
      SetColor (White);
      SetTextStyle (SmallFont,HorizDir,4);
		AlignX := w div 2 - TextWidth(BtnTxt) div 2;
		AlignY := h div 2 - TextHeight(BtnTxt) div 2 - 1;
      If AlignY < 3 then
         AlignY := 3;
		OutTextXY (AlignX,AlignY,BtnTxt);
      SetViewPort (0,0,GetMaxX,GetMaxY,ClipOn);
	end;

procedure ButtonObject.Create (Ptx,Pty,Wd,Ht,Color:integer;
								       TxtLabel:STR40;
                               Command:ProcType);
	begin
		x := Ptx;
		y := Pty;
		w := Wd;
		h := Ht;
		c := Color;
		BtnTxt := TxtLabel;
      NumButtons := NumButtons+1;
		Draw;
      Define (x,y,x+w,y+h,Command);
	end;

procedure ButtonObject.HiLite;
   const
      depth = 3;
   begin
		SetViewPort (x,y,x+w,y+h,ClipOn);
      if selected then
		   SetColor (Black)
      else
         SetColor (White);
      selected := NOT selected;
      HideMouse;
		Rectangle (0,0,w,h);
		Rectangle (2*depth,depth,w-2*Depth,h-Depth);
      Line (0,0,2*depth,depth);
		Line (0,h,2*depth,h-depth);
		Line (w,0,w-2*depth,depth);
		Line (w,h,w-2*depth,h-depth);
      ShowMouse;
      SetViewPort (0,0,GetMaxX,GetMaxY,ClipOn);
   end;

procedure CButtonObject.Draw;
   var
      i,AlignX,AlignY : integer;
   const
      depth = 3;

   begin
		SetViewPort (x,y,x+w,y+h,ClipOn);
		SetFillStyle (SolidFill,C);
		Bar (0,0,w,h);
		SetColor (Black);
		Rectangle (0,0,w,h);
		Rectangle (2*depth,depth,w-2*Depth,h-Depth);
      Line (0,0,2*depth,depth);
		Line (0,h,2*depth,h-depth);
		Line (w,0,w-2*depth,depth);
		Line (w,h,w-2*depth,h-depth);
      SetColor (White);
		AlignX := w div 4+3;
		AlignY := h div 4+1;
      Line(AlignX,h div 2,AlignX-5+w div 2,h div 2);
      If p = plus then
         Line(w div 2,AlignY,w div 2,AlignY-2+h div 2);
      SetViewPort (0,0,GetMaxX,GetMaxY,ClipOn);
	end;

procedure CButtonObject.Create (Ptx,Pty,Wd,Ht,Color:integer;
                                pol: polarity;
                                Command : ProcType);
   begin
      x := Ptx;
      y := Pty;
      w := Wd;
      h := Ht;
      c := Color;
      p := pol;
      NumCButtons := NumCButtons+1;
      Draw;
      Define (x,y,x+w,y+h,Command);
   end;

procedure DetectHits;
   var
      i,d,mx,my: integer;

   begin
      repeat
         d := WaitForInput(EitherButton);
         if d<0 then
            begin
               GetMouseCoords(mx,my);
               for i := 1 to NumButtons do
               begin
                  with Button[i] do
                     if MouseInBox (l,t,r,b,mx,my) then
                        begin
                           Button[i].HiLite;
                           Cmd;
                           Button[i].HiLite;
                        end;
               end;
               for i := 1 to NumCButtons do
               begin
                  with CButton[i] do
                     if MouseInBox (l,t,r,b,mx,my) then
                        begin
                           CButton[i].HiLite;
                           Cmd;
                           CButton[i].HiLite;
                        end;
               end;
            end;
         until FALSE;
      end;

Begin
   NumButtons := 0;
   NumCButtons := 0;
End.
